#include <u.h>
#include <libc.h>
#include <ctype.h>
#include <bio.h>
#include <ndb.h>

/*	connect ver.1.1a	*/

int vflag = 0;
int Kflag = 0;

void
usage(void)
{
	fprint(2,"usage: connect [-6vK] addr\n"
	"	-6: IPv6 connection"
	"	-v: verbose\n"
	"	-K: Kill the process that is waitng for response from remote server\n"
	"example: tcp!HOST!80\n");
	exits("usage");
}

void
kill(int pid)
{
	char buf[128];
	int fd;
	snprint(buf, sizeof(buf),"/proc/%d/note",pid);
	fd = open(buf,OWRITE);
	write(fd,"kill",4);
	close(fd);
}

int
xfer(int fd0, int fd1)
{
	char buf[16384];
	int n,m;
	while((n = read(fd0,buf, sizeof(buf))) > 0){
		m = write(fd1, buf, n);
		if(m < 0)
			break;
	}
	write(fd1,"",0);
	return (n <= 0)?0:1;
}


void
connect(int fd0, int fd1, char *dest)
{	int data;
	char devdir[NETPATHLEN];
	int pid;
	int status;

	if(vflag)
		fprint(2, "dialing to %s\n", dest);
	data = dial(dest, 0, devdir, 0);
	if(data < 0)
		sysfatal("%r");
	if(vflag){
		NetConnInfo *cinfo;
		fprint(2, "connected to %s on %s\n", dest, devdir);
		cinfo = getnetconninfo(nil, data);
		fprint(2, "local %s\n", cinfo->laddr);
	}
	switch((pid=rfork(RFPROC|RFFDG|RFMEM))){ /* assign = */
	case -1:
		perror("connect");
		exits("fork");
	case 0: // local to remote
		sleep(100); // wait for my partner to be ready
		status = xfer(fd0,data);
		if(vflag)
			fprint(2, status?"callee closed\n":"caller closed\n");
		if(Kflag)
			kill(getppid());
		exits(0);
	default: // remote to local
		status = xfer(data,fd1);
		if(vflag)
			fprint(2, status?"caller closed\n":"callee closed\n");
		kill(pid);
		exits(0);
	}
}

void
main(int argc, char *argv[])
{
	int ipver = 4;
	int n;
	char *p,*q;
	Ndbtuple *t, *nt;
	char *netroot = "/net";
	char *addr;
	char *args[3];

	ARGBEGIN{
	case '6': ipver = 6; break;
	case 'v': vflag = 1; break;
	case 'K': Kflag = 1; break;
	default: usage();
	}ARGEND

	if(argc != 1)
			usage();
	addr = strdup(argv[0]); // to be scratched
	n = getfields(addr,args,3,0,"!");
	if(n != 3)
		usage();
	if(strchr(args[1],'.'))
		p = csgetvalue(netroot, "dom", args[1], "ip", &t);
	else
		p = csgetvalue(netroot, "sys", args[1], "ip", &t);
	if(!p)
		sysfatal("bad address");
	if(*args[0] != '/')
		netroot = smprint("%s/%s",netroot,args[0]);


	p = nil;
	for(nt = t; nt != nil; nt = nt->entry) {
		/* note: csgetvalue() returns all attr even if "ip" is specified */
		if(strcmp(nt->attr, "ip") == 0) {
			p = netmkaddr(nt->val, netroot, args[2]);
			q = strchr(p,'!');
			if(ipver == 4 && strchr(q,'.'))
				break;
			if(ipver == 6 && strchr(q,':'))
				break;
			// free(p); don't free(p). look dial(2)
		}
		p = nil;
	}

	if(!p)
		sysfatal("bad address");

	free(netroot);
	ndbfree(t);
	connect(0,1,p);
	exits(nil);
}
